/* jshint undef: true */
/* global plugin, require, com, java, org, module */

var facade = require("facade");
var AtomicBoolean = java.util.concurrent.atomic.AtomicBoolean;
var RateLimiter = com.google.common.util.concurrent.RateLimiter;
var PluginProxy = com.cattura.agents.configuration_module.model.proxies.PluginProxy;
var LOGGER = plugin.info().logger;

/*
 * Throttle requests to log downloader plugin. We don't need to be dumping logs
 * if we are already doing so, and we don't want to do it too often.
 */
var currentlyDumpingLogs = new AtomicBoolean(false);
var logDumpLimiter = RateLimiter.create(1 / 600); // Limit to one log dump per ten minutes

function createAttachmentForIncident ()
{
    var pluginProxy = facade.retrieveProxy(PluginProxy.NAME);
    var logDownloaderPluginInfo = pluginProxy.getPluginInfo("log-downloader-plugin").orNull();
    if (!logDownloaderPluginInfo)
    {
        LOGGER.warning("Was unable to generate info dump for incident because log downloader plugin is unavailable.");
        return null;
    }
    
    if (!currentlyDumpingLogs.compareAndSet(false, true))
    {
        LOGGER.warning("Was unable to generate info dump for incident because log downloader plugin is currently dumping logs.");
        return null;
    }    
    
    try
    {
        if (!logDumpLimiter.tryAcquire())
        {
            LOGGER.warning("Was unable to generate info dump for incident because we have exceeded the rate for log dumping.");
            return null;
        }    
        
        var service = logDownloaderPluginInfo.getServicesByName("createLogArchive").get(0);
        return service['invoke(java.util.Map)']({
            params: {
                "includeExceptions": false
            }
        });
    }
    finally
    {
        currentlyDumpingLogs.set(false);
    }
}

module.exports = createAttachmentForIncident;